/*//////////////////////////////////////////////////////////////*\
**||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||**|
**---------------------------------------------------------------/
** Zenerd Standard Library 
**  Copyright (C) 2000-2002 by Zenerd.com
**
** Component: Time Line Editor Interface
**
** Date     : October 6, 2001
**
** Author(s): Jeffrey M. Barber
**
** Notes    : LITTLE to NO ERROR CHECKING
**          : <Beta>
**          : Uses Keith Rule's MemDC
**
**]=================================================================]
**This software is provided "AS IS," without a warranty of any kind.)
**	IN NO EVENT WILL ZENERD OR ITS LICENSORS BE LIABLE FOR ANY      )
**	LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,/
**	CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,                /
**---------------------------------------------------------------\
**||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||**|
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

/*//////////////////////////////////////////////////////////////*\
** Headers
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#include "stdafx.h"
#include "ZTimeAlloc.h"
#include "TimeLineEditor.h"
#include "ZTimeAllocDoc.h"
#include <math.h>
#include "MemDC.h"

/*//////////////////////////////////////////////////////////////*\
** MACROS
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CTimeLineEditor, CScrollView)

/*//////////////////////////////////////////////////////////////*\
** Automatic Object State Management (C/CC/D)'tors
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

CTimeLineEditor::CTimeLineEditor()
{
	TimeMax = 60000;
	SpanLeftMargin = 75;
	SpanRightMargin = 5;
	SpanHeight = 26;
	InnerBorder = 2;
	OuterOverlap = -1;
	HandleWidth = 8;
	TextMargin = 5;
	State = '0';
	TimeMinRes = 250;
	IdealMajorTickPix = 50;
	HeaderTop = 36;
	TickFreqMinor = 100;
	TickFreqMajor = 1000;
	TickMajorHeight = HeaderTop - 10;
	TickMinorHeight = HeaderTop - 20;
	SpanTopMargin   = HeaderTop + 2;
	ScaleToWindow = false;
	Editing = false;
	SnapToMinor = true;
	CheckWidth = 12;
	dSnap = false;
	dScale = false;
}

CTimeLineEditor::~CTimeLineEditor()
{
}

/*//////////////////////////////////////////////////////////////*\
** Message Map
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

BEGIN_MESSAGE_MAP(CTimeLineEditor, CScrollView)
	//{{AFX_MSG_MAP(CTimeLineEditor)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_ERASEBKGND()
	ON_WM_RBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/*//////////////////////////////////////////////////////////////*\
** Message Handlers: Basic
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void CTimeLineEditor::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	UpdateSizes();
}

// since I use Keith's Rule CMemDC, this would create flickering
//  and I already draw on the entire full screen
BOOL CTimeLineEditor::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

int CTimeLineEditor::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CScrollView::OnCreate(lpCreateStruct) == -1)
		return -1;

	// Create the CEdit, for editing the values in the spans
	Editor.Create(WS_VISIBLE, CRect(1,1,20,20), this, 1); // maybe change the id later since I don't know if that ID is in use
	// it shall be hidden
	Editor.ShowWindow(SW_HIDE);
	Editor.UpdateWindow();
	return 0;
}

/*//////////////////////////////////////////////////////////////*\
** Message Handlers: Mouse
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/
// Left Mouse Buttom Functions:

// relates to selection and checkboxes
void CTimeLineEditor::OnLButtonDown(UINT nFlags, CPoint point) 
{
	State = '0'; // reset the state
	CPoint M;
	M = point + GetScrollPosition(); // augment the point to rendering space;
	if(GetSnapRect().PtInRect(M)) // did I toggle in the Snap (custom checkbox)
		dSnap = true;
	if(GetScaleRect().PtInRect(M)) // did I toggle the Scale (custom checkbox)
		dScale = true;
	int res = TestPos(M, TempSpan, TempSlice); //Did I click on something imporant
	if(res>0)
	{
		switch(res)
		{
		case 1:
			State = 'l'; // change the state to the left handle
			break;
		case 2:
			State = 'r'; // change the state to the right handle
			break;
		case 3:
			TempOffset = GetTimeFromX(M.x) -  TempTime; // Record the offset, so we can move the bar along at the point we click
			State = 'b'; // change the state
			break;
		default:
			break;	
		}
	}
	KillEdit(); // If we were editing, kill it
	CScrollView::OnLButtonDown(nFlags, point);
}

void CTimeLineEditor::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CPoint M = point + GetScrollPosition(); // augment the point to rendering space;
	if(dSnap&&GetSnapRect().PtInRect(M)) // if we are still in the Snap, toggle the value
		SnapToMinor = !SnapToMinor; 
	if(dScale&&GetScaleRect().PtInRect(M)) // If we are still in the Scale, toggle the value
		ScaleToWindow = !ScaleToWindow;

	// reset the state variables for the snap and scale
	dSnap = false;
	dScale = false;
	
	// if we had a state, save the changes and reset state
	if(State!='0') 
	{
		WriteChange();
		State = '0';
	}
	TempSlice = -1;
	TempSpan = -1;
	KillEdit(); // if we were editing it, kill it
	Refresh();
	CScrollView::OnLButtonUp(nFlags, point);
}

void CTimeLineEditor::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument(); // uplink
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	CPoint M = point + GetScrollPosition(); // augment the point to rendering space;
	int res = TestPos(M, TempSpan, TempSlice); // test it
	if(TempSpan >= 0 && TempSpan < TimeLine->GetSpanCount()) // are we in a span
	{
		if(res==4) // yes
		{
			MakeSlice(TempSpan, GetTimeFromX(M.x), 1000, 0.0); // make a temp span 1 second long
			Refresh();
		}
	}
	CScrollView::OnLButtonDblClk(nFlags, point);
}

// Mouse Movement

void CTimeLineEditor::OnMouseMove(UINT nFlags, CPoint point) 
{
	CPoint M = point + GetScrollPosition(); // augment the point
	if(Editing)
	{
		CRect EditRect = GetSliceRect(EditSpan, TempTime,TempLength); // Get the rect of the slice to determine if the mouse goes outside it
		ExpandRect(EditRect); // make a rect big enough for mouse-error
		if(EditRect.PtInRect(M)==FALSE)
		{
			KillEdit(); // kill it, we left the boundries
			return;
		}
	}
	// copy the variables
	int TempTimeP = TempTime;
	int TempEndP = TempEnd;
	int TempLengthP = TempLength;

	// snap the End
	Snap(TempEndP);

	// how the it should update
	switch(State)
	{
		case 'l': // if the left handle is moved
				TempTimeP = GetTimeFromX(M.x);
				Snap(TempTimeP); // snap it
				TempLengthP = TempEndP - TempTimeP;
			break;
		case 'r': // if the right handle is moved
				TempLengthP = (abs(GetTimeFromX(M.x) - TempTimeP) + (GetTimeFromX(M.x) - TempTimeP))/2;
			break;
		case 'b': // if the whole is moved
				TempTimeP = GetTimeFromX(M.x) - TempOffset;
				TempEndP = TempTimeP + TempLengthP;
			break;
		default:
			break;
	}
	// the left mouse button states
	if(State=='l'||State=='r'||State=='b')
	{
		// error correcting
		FixBoundErrors(TempTimeP, TempLengthP, TempEndP);

		// grab the time line
		CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
		ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();

		// did I collide with anyone
		int CollideWith = TimeLine->isCollision(TempSpan, TempSlice, TempTimeP, TempLengthP);

		// I did not, therefore it is ok
		if(CollideWith==-1)
		{
			// if it didn't move, we change the Offset Value
			if(TempTime==TempTimeP && TempLength == TempLengthP)
			{
				TempOffset = GetTimeFromX(M.x) - TempTime;
				if(TempOffset<0) TempOffset = 0;
				if(TempOffset>(TempTime+TempLength)) TempOffset = (TempTime+TempLength);
			}

			// no collision, thus restore the variables
			TempTime = TempTimeP;
			TempLength = TempLengthP;
			TempEnd = TempEndP;

			// snap them into place
			Snap(TempTime);
			Snap(TempLength);
			Snap(TempEnd);
		} // I do, but can I manipulate it for better control
		else
		{
			// Whom do I collide with?
			int CStart, CLen;
			double X; // Temp Value
			TimeLine->GetSlice(TempSpan, CollideWith, CStart, CLen, X);

			// What side am I on
			X = GetTimeFromX(M.x);
			X-= (CStart + CLen / 2);

			// how should I "snap" it to the colliding one
			switch(State)
			{
				case 'b':
					if(X>=0) TempTimeP = abs(CStart + CLen + 1);
					if(X<0) TempTimeP = abs(CStart - TempLengthP - 1);
					break;
				case 'l':
					TempTimeP = CStart + CLen + 1;
					break;
				case 'r':
					TempLengthP = CStart - TempTimeP - 1;
					break;
			}
			// error correcting
			FixBoundErrors(TempTimeP, TempLengthP, TempEndP);
			// do I collide with the new one?
			if(TimeLine->isCollision(TempSpan, TempSlice, TempTimeP, TempLengthP)==-1)
			{
				// if it didn't move, we change the Offset Value
				if(TempTime==TempTimeP && TempLength == TempLengthP)
				{
					TempOffset = GetTimeFromX(M.x) - TempTime;
					if(TempOffset<0) TempOffset = 0;
					if(TempOffset>(TempTime+TempLength)) TempOffset = (TempTime+TempLength);
				}
				// restote
				TempTime = TempTimeP;
				TempLength = TempLengthP;
				TempEnd = TempEndP;
				// snap
				Snap(TempTime);
				Snap(TempLength);
				Snap(TempEnd);
			} // nothing left to do
		}
		Refresh();
	}
	// no state, highlight 
	if(State=='0')
	{
		// since TestPos is a State Driven function, it must be in the if statement
		int res = TestPos(M, TempSpan, TempSlice);
		MouseTime = GetTimeFromX(M.x);
		if(res>=0)
		{
		//	Refresh();
		}
	}
	CScrollView::OnMouseMove(nFlags, point);
}

// Right Mouse Button Activity
// for dealing with editing
void CTimeLineEditor::OnRButtonUp(UINT nFlags, CPoint point) 
{
	CPoint M = point + GetScrollPosition(); // augment
	int res = TestPos(M,TempSpan,TempSlice); // test it
	if(res==3) // we are over a slice
	{
		if(TempSpan == EditSpan && TempSlice == EditSlice) // if we are stll there
		{
			InitEdit(); // edit the weight
		}
	}
	CScrollView::OnRButtonUp(nFlags, point);
}

void CTimeLineEditor::OnRButtonDown(UINT nFlags, CPoint point) 
{
	KillEdit(); // kill the editor
	CPoint M = point + GetScrollPosition(); // augment
	int res = TestPos(M,TempSpan,TempSlice); // test

	// Store the Values when we clicked
	EditSpan = TempSpan;
	EditSlice = TempSlice;
	CScrollView::OnRButtonDown(nFlags, point);
}

/*//////////////////////////////////////////////////////////////*\
** Message Handlers: Rendering
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void CTimeLineEditor::OnDraw(CDC* pDCZ)
{
	// Variables
	CRect BckGr;
	CRect dT;
	CFont tLabelO;
	CFont tLabelA;
	CFont tWeight;
	CString MyOwner;
	CString MyAction;
	CString TempOwner = "";
	CString WeightVal;
	int i;

	// uplink to document
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	
	// enable CMemDC
#ifdef USE_MEMDC
	CMemDC memDC(pDCZ);
	CMemDC * pDC = & memDC;
#else
	CDC * pDC = pDCZ;
#endif

	// Render the Top Part of the BackGround
	BckGr = GetBckTopRect();
	drawBackground(pDC,BckGr);

	// Render the Bottom Part of the Background
	BckGr = GetBckBottomRect();
	drawBackground(pDC,BckGr);

	// update the scroll bars
	UpdateSizes();

	// make the fonts
	UseFont(tLabelO,14);
	UseFont(tLabelA,12);
	UseFont(tWeight,16);

	// Number of Spans
	int NumSpans = TimeLine->GetSpanCount();

	// Select A Font
	pDC->SelectObject(&tLabelA);

	for(i= 0 ;i < NumSpans;i++)
	{
		// draw the Span
		drawSpan(pDC, i);

		// Get Owners from Document
		MyOwner = pDoc->GetOwnerName  (TimeLine->GetOwner(i) );
		MyAction = pDoc->GetActionName(TimeLine->GetAction(i));
		// Make it so the Owner's Group together
		if(MyOwner!=TempOwner)
			TempOwner = MyOwner;
		else
			MyOwner = "";
		
		// Get the Label Rectangle
		dT = GetLabelRect(i);

		// Draw the Dual Label
		drawDualLabelText(pDC, dT, tLabelO, MyOwner, tLabelA, MyAction);

		// Number of Slices for this Span
		int NumSlice = TimeLine->GetSliceCount(i);
		for(int j=0;j<NumSlice;j++)
		{
			// get the slice
			int start, length;
			double weight;
			TimeLine->GetSlice(i,j,start,length, weight);
			// we are moving the left handle
			if(State=='l' && TempSpan==i && TempSlice==j)
			{
				start = TempTime;
				length = TempLength;
			}
			// we are moving the right handle
			else if(State=='r' && TempSpan==i && TempSlice==j)
			{
				length = TempLength;
			}
			// we are moving the entire slice
			else if(State=='b' && TempSpan==i && TempSlice==j)
			{
				start = TempTime;
			}
			// draw the weight value
			WeightVal.Format("%.2f",(float) weight);; // getFromDocument
				
			dT = GetSliceRect(i, start, length);
			//modify the Text Span Rectangle
				dT.left += TextMargin;
				dT.top+=2;
				pDC->SelectObject(&tWeight);
			// if I am however over this one, highlight it
			if(TempSpan == i && TempSlice == j)
			{
				drawTimeSplice(pDC, i, start, length, GetSysColor(COLOR_HIGHLIGHT));
				pDC->SetBkColor(GetSysColor(COLOR_HIGHLIGHT));
				pDC->SetTextColor(GetSysColor(COLOR_HIGHLIGHTTEXT));
			}
			else
			{
				// not highlighted
				drawTimeSplice(pDC, i, start, length, GetSysColor(COLOR_WINDOW));
				pDC->SetBkColor(GetSysColor(COLOR_WINDOW));
				pDC->SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
			}
			// render the text
			pDC->DrawText(WeightVal,dT,0);
		}
	}

	// draw the top portion
	drawTop(pDC);
	// Draw Check Box
	drawCheckBox(pDC, GetSnapRect(), CString("Snap In"), SnapToMinor);
	drawCheckBox(pDC, GetScaleRect(), CString("Scaling"), ScaleToWindow);

}

/*//////////////////////////////////////////////////////////////*\
** Debug
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#ifdef _DEBUG
void CTimeLineEditor::AssertValid() const
{
	CScrollView::AssertValid();
}

void CTimeLineEditor::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}
#endif //_DEBUG

/*//////////////////////////////////////////////////////////////*\
** Scalable Functions Processing
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

int CTimeLineEditor::GetSpanWidth()
{
	CRect myR;
	GetClientRect(myR);
	if(ScaleToWindow)
	{
		return myR.Width() - SpanLeftMargin - SpanRightMargin;
	}
	return GetNumberMajorTicks() *  IdealMajorTickPix;
}

int CTimeLineEditor::GetTotalHeight()
{
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	return GetSpanTop( TimeLine->GetSpanCount() );
}

int CTimeLineEditor::GetNumberMajorTicks()
{
	return TimeMax / TickFreqMajor;
}

/*//////////////////////////////////////////////////////////////*\
** Positioning Within the Scale
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

int CTimeLineEditor::GetSpanTop(int Span)
{
	return Span * (SpanHeight + InnerBorder) + SpanTopMargin;
}

int CTimeLineEditor::GetTimeToX(int Time)
{
	return GetSpanWidth() * Time / TimeMax;
}

int CTimeLineEditor::GetSliceLeft(int Time)
{
	return SpanLeftMargin + GetTimeToX(Time);
}

int CTimeLineEditor::GetSliceRight(int Time, int Length)
{
	return SpanLeftMargin + GetTimeToX(Time+Length);
}

void CTimeLineEditor::Snap(int &Val)
{
	// if we are snaping, snap it to the minor ticks
	if(SnapToMinor)
	{
		// basically, a integer-base<TickFreqMinor> truncate 
		Val /= TickFreqMinor;
		Val *= TickFreqMinor;
	}
}

/*//////////////////////////////////////////////////////////////*\
** Inverse Positioning Within the Scale
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

// what span is this y value in
int CTimeLineEditor::DetermineSpan(int y)
{
	if(y<=SpanTopMargin) return -1;
	if(y>=GetTotalHeight()) return -1;
	int u = (y - SpanTopMargin)/(SpanHeight + InnerBorder);
	return u;
}

// what is the time at this x at
int CTimeLineEditor::GetTimeFromX(int x)
{
	if(x<SpanLeftMargin) return 0;
	if(x>SpanLeftMargin + GetSpanWidth()) return TimeMax;
	return (x - SpanLeftMargin) * TimeMax / GetSpanWidth();
}

/*//////////////////////////////////////////////////////////////*\
** Testing Positioning
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

int CTimeLineEditor::TestPos(CPoint M, int &Span, int &Slice)
{
	return TestPos(M.x, M.y, Span, Slice);
}

int CTimeLineEditor::TestPos(int x, int y, int &Span, int &Slice)
{
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	CPoint P(x,y);
	Span = DetermineSpan(y);
	if(Span>=0)
	{
		if(GetSpanEdge(Span).PtInRect(P)==TRUE)
		{
			TempSlice = -1;
			for(int k=0;k<TimeLine->GetSliceCount(Span);k++)
			{
				int time;
				int len;
				TimeLine->GetSlice(Span,k,time,len, TempWeight);
				TempTime = time;
				TempLength = len;
				TempEnd = TempTime + TempLength;
				Snap(TempEnd);
				if(GetHandleLeft (Span, time, len).PtInRect(P)==TRUE)
					{ TempSlice = k; return 1;}
				if(GetHandleRight(Span, time, len).PtInRect(P)==TRUE)
					{ TempSlice = k; return 2;}
				if(GetSliceRect  (Span, time, len).PtInRect(P)==TRUE)
					{ TempSlice = k; return 3;}
			}
			return 4;
		}
		if(GetLabelRect(Span).PtInRect(P)==TRUE)
		{
			return 5;
		}
		return 0;
	}
	return -1;
}

/*//////////////////////////////////////////////////////////////*\
** Data Functions
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void CTimeLineEditor::WriteChange()
{
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	int start, length;
	double weight;
	TimeLine->GetSlice(TempSpan,TempSlice,start,length, weight);
	if(State=='l')
	{	
		start	= TempTime;
		length	= TempEnd - TempTime;
	}
	else if(State=='r')
	{	length	= TempLength;	}
	else if(State=='b')
	{	start	= TempTime;		}
	if(Editing)
		weight = TempWeight;
	TimeLine->Edit(TempSpan, TempSlice, start, length, weight);
}

void CTimeLineEditor::FixBoundErrors(int &TT, int &TL, int &TE)
{
	if(TT<0) TT = 0;
	if(TT+TL>=TimeMax) TT = TimeMax - TL;
	if(TL < TimeMinRes) TL = TimeMinRes;
	if(TT > (TE - TimeMinRes) ) TT = TE - TimeMinRes;
}

void CTimeLineEditor::MakeSlice(int Span, int Time, int Length, double Weight)
{
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	TimeLine->AddTimeSlice(Span, Time, 1000, Weight);
}

void CTimeLineEditor::SetTimeMax(int Max)
{
	TimeMax = Max;
}

void CTimeLineEditor::Refresh()
{
	InvalidateRect(NULL);
}

/*//////////////////////////////////////////////////////////////*\
** Rendering Helper Functions
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void CTimeLineEditor::drawSpan(CDC* pDC, int Span)
{
	CRect BckGr;
	GetClientRect(BckGr);
	BckGr.top = GetSpanTop(Span);
	BckGr.bottom = BckGr.top + SpanHeight + InnerBorder;
	BckGr.left += GetScrollPosition().x;
	BckGr.right += GetScrollPosition().x;
	drawBackground(pDC, BckGr);

	CRect SpanEdge = GetSpanEdge(Span);
	pDC->FillSolidRect(SpanEdge, GetSysColor(COLOR_3DLIGHT));
	pDC->DrawEdge(SpanEdge, EDGE_SUNKEN , BF_RECT);
}

void CTimeLineEditor::drawTimeSplice(CDC *pDC, int Span, int Time, int Length, COLORREF Color)
{
	CRect Slice = GetSliceRect(Span, Time, Length);
	pDC->FillSolidRect(Slice, Color);
	pDC->DrawEdge(Slice, EDGE_SUNKEN , BF_TOP|BF_BOTTOM);
	drawHandle(pDC,GetHandleRight(Span, Time, Length));
	drawHandle(pDC,GetHandleLeft (Span, Time, Length));
}

void CTimeLineEditor::drawHandle(CDC *pDC, CRect Handle)
{
	pDC->FillSolidRect(Handle, GetSysColor(COLOR_BTNFACE));
	pDC->DrawEdge(Handle, EDGE_RAISED , BF_RECT);
	CRect Interior = GetHandleInterior(Handle);
	pDC->DrawEdge(Interior, EDGE_SUNKEN, BF_RECT);
}

void CTimeLineEditor::drawTop(CDC *pDC)
{
	CRect Top;
	GetClientRect(&Top);
	Top.bottom = Top.top + HeaderTop;
	Top.left += GetScrollPosition().x;
	Top.right += GetScrollPosition().x;
	Top.top += GetScrollPosition().y;
	Top.bottom += GetScrollPosition().y;

	pDC->FillSolidRect(Top, RGB(255,255,255));
	pDC->DrawEdge(Top, EDGE_SUNKEN, BF_BOTTOM);
	CRect Tick;
	Tick.top = GetScrollPosition().y;
	Tick.bottom = Tick.top + HeaderTop;
	Tick.left = GetSliceLeft(0);
	Tick.right = Tick.left + GetSpanWidth();
	CPen MinorTicks;
	CPen MajorTicks;
	MinorTicks.CreatePen(PS_SOLID  ,1 , RGB(200,200,200) );
	MajorTicks.CreatePen(PS_SOLID  ,1 , RGB(100,100,100) );
	int x;
	pDC->SelectObject(&MinorTicks);
	int TX;
	for(x = 0; x<TimeMax; x+=TickFreqMinor)
	{
		TX = GetSliceLeft(x);
		pDC->MoveTo(TX,GetScrollPosition().y);
		pDC->LineTo(TX,GetScrollPosition().y+TickMinorHeight);
	}

	pDC->SelectObject(&MajorTicks);
	for(x = 0; x<TimeMax; x+=TickFreqMajor)
	{
		TX = GetSliceLeft(x);
		pDC->MoveTo(TX,GetScrollPosition().y);
		pDC->LineTo(TX,GetScrollPosition().y+TickMajorHeight);
	}
	CFont tFont;
	tFont.CreateFont(12, 0, 0, 0,0, 0,0,0,0,0,0,0,0,"Arial");
	pDC->SelectObject(&tFont);
	pDC->SetBkColor(RGB(255,255,255));
	pDC->SetTextColor(RGB(255,0,0));
	drawTime(pDC, TempTime);
	pDC->SetTextColor(RGB(0,0,255));
	drawTime(pDC, TempTime+TempLength);
}

void CTimeLineEditor::drawTime(CDC *pDC, int Time)
{
	CRect tRect = GetTimeRect(Time);
	double dTime = Time / 1000.0;
	CString value;
	value.Format(" %.2f secs",(float)dTime);
	pDC->DrawText(value, tRect, DT_BOTTOM);
	pDC->MoveTo(tRect.left,tRect.top);
	pDC->LineTo(tRect.left,tRect.bottom);
}

void CTimeLineEditor::drawBackground(CDC *pDC, CRect & BckGr)
{
	pDC->FillSolidRect(BckGr, GetSysColor(COLOR_3DFACE));
}

void CTimeLineEditor::drawCheckBox(CDC *pDC, CRect Chk, CString label, bool Fill)
{
	if(Fill)
		pDC->FillSolidRect(Chk,RGB(0,0,0));
	pDC->SetBkColor(RGB(255,255,255));
	pDC->SetTextColor(RGB(0,0,0));
	pDC->DrawEdge(Chk, EDGE_RAISED, BF_RECT);
	CRect ChkT = Chk;
	ChkT.left = Chk.right + 2;
	ChkT.right = Chk.left + 100;
	pDC->DrawText(label, ChkT, DT_TOP);
}

void CTimeLineEditor::drawDualLabelText(CDC *pDC, CRect Box, CFont &FA, CString labelMajor, CFont &FB, CString labelMinor)
{
	pDC->SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
	pDC->SetBkColor(GetSysColor(COLOR_3DFACE));
	pDC->SelectObject(&FA);
	pDC->DrawText(labelMajor ,Box ,DT_LEFT  |DT_SINGLELINE|DT_TOP  );
	pDC->SelectObject(&FB);
	pDC->DrawText(labelMinor ,Box ,DT_BOTTOM|DT_SINGLELINE|DT_RIGHT);
}

void CTimeLineEditor::UpdateSizes()
{
	CSize sizeTotal;
	sizeTotal.cy = GetTotalHeight();
	sizeTotal.cx = GetSpanWidth() + SpanLeftMargin + SpanRightMargin;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

void CTimeLineEditor::UseFont(CFont &font, int height)
{
	font.CreateFont(height, 0, 0, 0,0, 0,0,0,0,0,0,0,0,"Arial");
}



/*//////////////////////////////////////////////////////////////*\
** Editing the Weighted Values
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

void CTimeLineEditor::InitEdit()
{
	CRect EditRect = GetSliceRect(EditSpan, TempTime,TempLength);
	CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
	ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
	CString val;
	val.Format("%.2f",TempWeight);
	Editor.SetWindowText(val);
	EditRect.left+=2;
	EditRect.right-=2;
	EditRect.top+=2;
	EditRect.bottom-=2;
	Editor.MoveWindow(&EditRect, TRUE);
	Editor.ShowWindow(SW_SHOW);
	Editor.SetSel(0,-1);
	Editor.UpdateWindow();
	Editing = true;
}

void CTimeLineEditor::KillEdit()
{
	if(Editing==true)
	{
		CZTimeAllocDoc * pDoc = (CZTimeAllocDoc *) GetDocument();
		ZTimeLine * TimeLine = pDoc->GetCurrentTimeLine();
		Editor.ShowWindow(SW_HIDE);
		Editor.UpdateWindow();
		CString val;
		Editor.UpdateData();
		Editor.GetWindowText(val);
		TempWeight = atof(val);
		TempSpan = EditSpan;
		TempSlice = EditSlice;
		Editor.SetWindowText("");
		WriteChange();
	}
	Editing = false;
}


/*//////////////////////////////////////////////////////////////*\
** Rectangle Processing
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

CRect CTimeLineEditor::GetSpanEdge(int Span)
{
	CRect SpanEdge;
	SpanEdge.left = SpanLeftMargin;
	SpanEdge.right = SpanEdge.left + GetSpanWidth();
	SpanEdge.top = GetSpanTop(Span);
	SpanEdge.bottom = SpanEdge.top + SpanHeight;
	return SpanEdge;
}

CRect CTimeLineEditor::GetSliceRect(int Span, int Time, int Length)
{
	CRect Slice;
	Slice.left = GetSliceLeft(Time) + HandleWidth;
	Slice.right = GetSliceRight(Time, Length) - HandleWidth;
	Slice.top = GetSpanTop(Span) + InnerBorder;
	Slice.bottom = Slice.top + SpanHeight - 2 * InnerBorder;
	return Slice;
}

CRect CTimeLineEditor::GetHandleLeft(int Span, int Time, int Length)
{
	CRect Handle;
	Handle.left =  GetSliceLeft(Time);
	Handle.right =  Handle.left + HandleWidth;
	Handle.top = GetSpanTop(Span) - OuterOverlap;
	Handle.bottom = Handle.top + SpanHeight + 2 * OuterOverlap;
	return Handle;
}

CRect CTimeLineEditor::GetHandleRight(int Span, int Time, int Length)
{
	CRect Handle;
	Handle.right = GetSliceRight(Time, Length);
	Handle.left = Handle.right - HandleWidth;
	Handle.top = GetSpanTop(Span) - OuterOverlap;
	Handle.bottom = Handle.top + SpanHeight + 2 * OuterOverlap;
	return Handle;
}

CRect CTimeLineEditor::GetHandleInterior(CRect Handle)
{
	CRect Interior;
	Interior = Handle;
	Interior.left += InnerBorder;
	Interior.right -= InnerBorder;
	Interior.top += InnerBorder;
	Interior.bottom -= InnerBorder;
	return Interior;
}

CRect CTimeLineEditor::GetLabelRect(int Span)
{
	CRect Rect;
	Rect.left = TextMargin;
	Rect.right = SpanLeftMargin - TextMargin;
	Rect.top = GetSpanTop(Span);
	Rect.bottom = Rect.top + SpanHeight;
	return Rect;
}

CRect CTimeLineEditor::GetSnapRect()
{
	CRect ret;
	ret.left   = InnerBorder;
	ret.right  = ret.left   + CheckWidth;

	ret.top    = InnerBorder + GetScrollPosition().y;
	ret.bottom = ret.top    + CheckWidth ;
	return ret;
}

CRect CTimeLineEditor::GetScaleRect()
{
	CRect ret;

	ret = GetSnapRect();

	ret.top    = ret.bottom+InnerBorder;
	ret.bottom = ret.top   +CheckWidth;

	return ret;
}

CRect CTimeLineEditor::GetBckTopRect()
{
	CRect BckGr;
	GetClientRect(BckGr);
	BckGr.top = 0;
	BckGr.bottom = SpanTopMargin;
	BckGr.left += GetScrollPosition().x;
	BckGr.right += GetScrollPosition().x;
	return BckGr;
}

CRect CTimeLineEditor::GetBckBottomRect()
{
	CRect BckGr;
	GetClientRect(BckGr);
	BckGr.top = GetTotalHeight();
	BckGr.bottom+= GetScrollPosition().y;
	BckGr.left += GetScrollPosition().x;
	BckGr.right += GetScrollPosition().x;
	if(BckGr.bottom<BckGr.top)
	{
		BckGr.bottom = BckGr.top + 1;
	}
	return BckGr;
}

CRect CTimeLineEditor::GetTimeRect(int Time)
{
	CRect tRect;
	tRect.left = SpanLeftMargin + GetTimeToX(Time);
	tRect.right = tRect.left + 100;
	tRect.top = GetScrollPosition().y + HeaderTop - 16;
	tRect.bottom = GetScrollPosition().y + HeaderTop - 4;
	return tRect;
}

void CTimeLineEditor::ExpandRect(CRect &SliceRect)
{
	SliceRect.left   -= SliceRect.Width();
	SliceRect.right  += SliceRect.Width();// yes, it changes, besides the point
	SliceRect.top    -= SliceRect.Height();
	SliceRect.bottom += SliceRect.Height();// yes, it changes, besides the point
}

BOOL CTimeLineEditor::OnScrollBy(CSize sizeScroll, BOOL bDoScroll) 
{
	// TODO: Add your specialized code here and/or call the base class
	Refresh();
	return CScrollView::OnScrollBy(sizeScroll, bDoScroll);
}

BOOL CTimeLineEditor::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll) 
{
	// TODO: Add your specialized code here and/or call the base class
	Refresh();
	return CScrollView::OnScroll(nScrollCode, nPos, bDoScroll);
}
