/*//////////////////////////////////////////////////////////////*\
**||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||**|
**---------------------------------------------------------------/
** Zenerd Standard Library 
**  Copyright (C) 2000-2002 by Zenerd.com
**
** Component: Time Line Editor Interface
**
** Date     : October 6, 2001
**
** Author(s): Jeffrey M. Barber
**
** Notes    : NO ERROR CHECKING
**          : Uses Keith Rule's MemDC
**
**]=================================================================]
**This software is provided "AS IS," without a warranty of any kind.)
**	IN NO EVENT WILL ZENERD OR ITS LICENSORS BE LIABLE FOR ANY      )
**	LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,/
**	CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES,                /
**---------------------------------------------------------------\
**||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||**|
**\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\*/

#if !defined(AFX_TIMELINEEDITOR_H__C7B56013_9349_4461_BABD_156E8EA6BBC0__INCLUDED_)
#define AFX_TIMELINEEDITOR_H__C7B56013_9349_4461_BABD_156E8EA6BBC0__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define USE_MEMDC

/////////////////////////////////////////////////////////////////////////////
// CTimeLineEditor view
class CTimeLineEditor : public CScrollView
{
protected:
	CTimeLineEditor(); // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(CTimeLineEditor)
private:

	// Rectangle Processing, for simpler Rendering procedure
	CRect	GetHandleInterior(CRect Handle);
	CRect	GetSpanEdge(int Span);
	CRect	GetHandleRight(int Span, int Time, int Length);
	CRect	GetHandleLeft(int Span, int Time, int Length);
	CRect	GetSliceRect(int Span, int Time, int Length);
	CRect	GetLabelRect(int Span);
	CRect	GetSnapRect();
	CRect	GetScaleRect();
	CRect	GetBckTopRect();
	CRect	GetBckBottomRect();
	CRect	GetTimeRect(int Time);
	void	ExpandRect(CRect & SliceRect);

	// Various Helper Functions
	void	UseFont(CFont &font, int height);
	void	UpdateSizes();

	// Position Functions for Scalable Rendering
	int		GetSliceLeft(int Time); // in Pixels
	int		GetSliceRight(int Time, int Length); // in Pixels
	int		GetTotalHeight();
	int		GetSpanTop(int Span);
	int		GetSpanWidth();
	int		GetNumberMajorTicks();
	int		GetTimeToX(int Time);
	
	// Test a mouse click, note: sets TempTime, TempLength, TempWeight
	int		TestPos(int x, int y, int &Span, int &Slice);
	int		TestPos(CPoint M, int &Span, int &Slice);
	void	FixBoundErrors(int &TT, int &TL, int &TE);

	// inverse position functions
	int		GetTimeFromX(int x);
	int		DetermineSpan(int y);

	// rendering functions
	void	drawTop(CDC *pDC);
	void	drawTime(CDC *pDC, int Time);
	void	drawSpan(CDC* pDC, int Span);
	void	drawTimeSplice(CDC *pDC, int Span, int Time, int Length, COLORREF Color);
	void	drawHandle(CDC *pDC, CRect Handle);
	void	drawCheckBox(CDC *pDC, CRect Chk, CString label, bool Fill);
	void	drawBackground(CDC *pDC, CRect & BckGr);
	void	drawDualLabelText(CDC *pDC, CRect Box, CFont &FA, CString labelMajor, CFont &FB, CString labelMinor);

	// write a change from Temps to the Document
	void	WriteChange();
	void	Snap(int &Val);
	void	KillEdit();
	void	InitEdit();

// Attributes
private:
	CEdit	Editor;

	double	TempWeight;

	int		TempSpan;
	int		TempSlice;
	int		TempTime;
	int		TempLength;
	int		TempEnd;
	int		TempOffset;
	int		MouseTime;
	int		EditSpan;
	int		EditSlice;

	char	State;
	
	bool	Editing;
	bool	dSnap;
	bool	dScale;

public:
	int		TimeMax;
	int		TimeMinRes;
	int		SpanHeight;
	int		TextMargin;
	int		SpanLeftMargin;
	int		SpanRightMargin;
	int		InnerBorder;
	int		OuterOverlap;
	int		HandleWidth;
	int		SpanTopMargin;
	int		TickFreqMinor;
	int		TickFreqMajor;
	int		HeaderTop;
	int		TickMajorHeight;
	int		TickMinorHeight;
	int		IdealMajorTickPix;
	int		CheckWidth;

	bool	SnapToMinor;
	bool	ScaleToWindow;
// Operations
public:
	void SetTimeMax(int Max);
	void MakeSlice(int Span, int Time, int Length, double Weight); // Insert a new slice
	void Refresh();
	
// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CTimeLineEditor)
	public:
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	virtual void OnInitialUpdate();     // first time after construct
	virtual BOOL OnScrollBy(CSize sizeScroll, BOOL bDoScroll = TRUE);
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CTimeLineEditor();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
	//{{AFX_MSG(CTimeLineEditor)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_TIMELINEEDITOR_H__C7B56013_9349_4461_BABD_156E8EA6BBC0__INCLUDED_)
